function [H,P,N,T,T_perm] = wbl_LRT (FRI_data,alpha,n_perm)
% wbl_LRT.m
% Compare two populations fit with Weibull distributions using a
% Likelihood-ratio Test to test Ho: b1 = b2 AND c1 = c2 against Ha: b1 ~= b2
% OR c1 ~= c2, where b and c are the maximum likelihood estimates for the
% scale and shape parameters, respectively, for each hypothesized
% population (1,2...n).

% The likelihood-ratio test statistic, T, is defined as:
% 	T =  2(logLike(H1) + logLike(H2)- logLike(Ho))
% If the true parameters of Ho and Ha are equal, then the distribution of T
% is approximately chi-square with n-2 degrees of freedom, where n is the
% number of observations in both populations.
% 
% If the parameter n_perm = 0, then:
% T is compared to a Chi-squared distribution with k degrees of
% freedom (where k = 2 = # of additional parameters in H1 relative to H0),
% and a p-value is derived. If p < 1-alpha, then H0 is rejected.
%
% If the parameter n_perm > 0, then the program follows the following rationale:
% In practice, the distribution of T under Ho is unknown, so we estimate
% it using a permutation test. Under the assumption of Ho, we select N1,
% N2, and N1+N2 samples from a permuted Ho dataset. We then calculate the
% likelihood ratio statistic in the same fashion as under Ha.
% We do this n_perm times and compute a new statistic T(j) for each
% permutation. Using the newly computed statistics (T(j), T(j+1)...
% T(n_perm))we can approximate the p-value to test Ho:
% p = 1/n_perm * (sum(T(j) > T)), where '(sum(T(j) >T))' represents the
% number of times a randomly generated permutation results in a log
% likelihood difference greater than the log likelihood difference observed
% in the original dataset. If the two distributions are truly different,
% then the statistics from the permutations, T(j), should be smaller
% than T most of the time.

%% CREATE VARIABLES AND SPACE FOR NEW VARIABLES
[n m] = size(FRI_data);     % Dimensions of incoming dataset, with n rows
                                % of fire return intervals, and m columns
                                % of populations
H = NaN(m-1,m-1);           % Space for hypothesis test results: 1 = reject
                                % 0 = fail to reject, assuming alpha
P = NaN(m-1,m-1);           % Space for p-value to go for each comparison
N = NaN(1,m);               % Space for N (sample size) of each population

%% COMPARE EACH POPULATION

for i = 1:m-1   % For each population, popn1
    disp (['popn. comparison ',num2str(i),' of ',num2str(m-1)]) % Display
                % the comparison being made on screen
for k = i+1:m   % For each population being compared to popn1, popn2
    % define the populations
    popn1 = FRI_data(FRI_data(:,i)>0,i);    % Population 1
    popn2 = FRI_data(FRI_data(:,k)>0,k);    % Population 2
    popn_Ho = [popn1; popn2];               % Combined population, Ho
    n1 = length(popn1);                     % N for population 1
    n2 = length(popn2);                     % N for population 2
    nHo = n1+n2;                            % N for Ho population

    % Estimate Weibull a and b parameters with maximum likelihood method
    param_popn1 = wblfit(popn1);    % MLE for Weibull b and c parameters
                                        % for popn1
    param_popn2 = wblfit(popn2);    % Same for popn2
    param_Ho = wblfit(popn_Ho);     % Same for popn_Ho

    % Compute negative log-likelihood values for each population
    NLL_popn1 = -1*wbllike(param_popn1,popn1);  % Negative log-likelihood
                                    % for popn1
    NLL_popn2 = -1*wbllike(param_popn2,popn2);  % Negative log-likelihood
                                    % for popn2
    NLL_popn_Ho = -1*wbllike(param_Ho,popn_Ho); % Negative log-likelihood
                                    % for popn_Ho
    % Calculate likelihood ratio statistic, T
    T = 2*(NLL_popn1 + NLL_popn2 - NLL_popn_Ho);

    if n_perm ==0 % If n_perm = 0, derive p-value from Chi^2 distribution
           % Compute likelihood ratio statistic, T
        T = 2*(NLL_popn1 + NLL_popn2 - NLL_popn_Ho);
        P_val = 1-chi2cdf(T,2);     % p-value derived from chi squared
                                    % distribution
        P(i,k-1) = P_val;   % Fill in P matrix with p-values
        if P_val < alpha
            H(i,k-1) = 1;   % If P is less than alpha, make H = 1
        else
            H(i,k-1) = 0;   % Else make H = 0
        end
    else    % If n_perm ~= 0, derive p-value from permutations
    % Calculate n_perm T(j) statistics to calculate p-value for each
        % comparison
    for j = 1:n_perm    % For each permutation, j
        perm = randperm(length(popn_Ho));    % Random permutation of the
            % combined dataste (popn_Ho)
        Ho_perm = popn_Ho(perm);    % Ho population for this permutation
            % is the permutations selected from popn_Ho
        popn1_perm = Ho_perm(1:length(popn1));  % popn1 for this
            % permutation is the first length(popn1) values from Ho_perm
        popn2_perm = Ho_perm(length(popn1)+1:length(Ho_perm));  % popn2 for
            % this permutation is the remaining values from Ho_perm

        % Compute negative log likelihood values for each population
        NLL_Ho_perm = -1*wbllike((wblfit(Ho_perm)),Ho_perm);    % Negative
            % log-likelihood values for Ho_perm
        NLL_popn1_perm = -1*wbllike((wblfit(popn1_perm)),popn1_perm);
            % Negative log-likelihood values for popn1_perm
        NLL_popn2_perm = -1*wbllike((wblfit(popn2_perm)),popn2_perm);
            % Negative log-likelihood values for popn2_perm

        T_perm(j) = 2*(NLL_popn1_perm + NLL_popn2_perm - NLL_Ho_perm);
            % T statistic for this permutation, j
    end
    % Find p-value for this comparison
    P_val = 1/n_perm * sum(T_perm > T); % P is the number of time T_perm is
        % greater than T, divided by the total number of permutations.

    P(i,k-1) = P_val;   % Fill in P matrix with p-values
    if P_val < alpha
        H(i,k-1) = 1;   % If P is less than alpha, make H = 1
    else
        H(i,k-1) = 0;   % Else make H = 0
    end
    end                 % End code to calculate p-value
    N(i) = n1;          % Fill in N with n of popn1
end
end
N(length(N)) = n2;      % Fill in the last value in N with n of popn2